package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.dao.TEExamSummaryMapper;
import com.qmth.themis.business.entity.TEExamSummary;
import com.qmth.themis.business.service.TEExamActivityService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.service.TEExamSummaryService;
import com.qmth.themis.business.service.ThemisCacheService;
import com.qmth.themis.business.util.UidUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Objects;
import java.util.Set;

/**
 * <p>
 * 考试信息统计表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-10-27
 */
@Service
public class TEExamSummaryServiceImpl extends ServiceImpl<TEExamSummaryMapper, TEExamSummary> implements TEExamSummaryService {
    private final static Logger log = LoggerFactory.getLogger(TEExamSummaryServiceImpl.class);

    @Resource
    TEExamActivityService teExamActivityService;

    @Resource
    ThemisCacheService themisCacheService;

    @Resource
    TEExamStudentService teExamStudentService;

    /**
     * 统计考试信息
     *
     * @param examId
     * @param examActivityId
     * @param roomCodeSet
     */
    @Override
    public void examSummary(Long examId, Long examActivityId, Set<String> roomCodeSet) {
        if (!CollectionUtils.isEmpty(roomCodeSet)) {
            for (String s : roomCodeSet) {
                //根据考试批次、场次、考场编码统计信息
                TEExamSummary teExamSummary = this.baseMapper.examSummary(examId, examActivityId, s);
                this.examSummaryCommon(examId, examActivityId, s, teExamSummary);
            }
            themisCacheService.updateExamSummaryCache(examId);
            themisCacheService.updateExamSummaryCache(examId, examActivityId);
        }
    }

    /**
     * 考试计算公用
     *
     * @param examId
     * @param examActivityId
     * @param s
     * @param teExamSummary
     */
    protected void examSummaryCommon(Long examId, Long examActivityId, String s, TEExamSummary teExamSummary) {
        if (Objects.nonNull(teExamSummary) && teExamSummary.getTotalCount().intValue() > 0) {
            teExamSummary.setExamId(examId);
            teExamSummary.setExamActivityId(examActivityId);
            teExamSummary.setRoomCode(s);
            ExamActivityCacheBean ac = teExamActivityService.getExamActivityCacheBean(examActivityId);
            long timestamp = System.currentTimeMillis();
            if (ac.getFinishTime() <= timestamp) {//当考试场次结束时间已过，缺考=全部应考-已完成考试
                teExamSummary.setAbsentCount(teExamSummary.getTotalCount() - teExamSummary.getFinishCount());
            } else {
                Integer absentCount = teExamStudentService.getAbsentCount(examId, examActivityId, s);
                teExamSummary.setAbsentCount(Objects.nonNull(absentCount) ? absentCount : 0);
            }
            TEExamSummary teExamSummaryDb = this.getOne(new QueryWrapper<TEExamSummary>().lambda()
                    .eq(TEExamSummary::getExamId, examId)
                    .eq(TEExamSummary::getExamActivityId, examActivityId)
                    .eq(TEExamSummary::getRoomCode, s));
            if (Objects.isNull(teExamSummaryDb)) {
                teExamSummaryDb = new TEExamSummary();
                BeanUtils.copyProperties(teExamSummary, teExamSummaryDb);
                teExamSummaryDb.setId(UidUtil.nextId());
            } else {
                Long id = teExamSummaryDb.getId();
                BeanUtils.copyProperties(teExamSummary, teExamSummaryDb);
                teExamSummaryDb.setId(id);
            }
            this.saveOrUpdate(teExamSummaryDb);
            themisCacheService.updateExamSummaryCache(examId, examActivityId, s);
        }
    }
}
